
/* CellWriterEngine implementation 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>

#include <cwds.h>
#include "CellWriterEngine.h"

/*
 * Constructor
 */
CellWriterEngine::CellWriterEngine(MainGraphicsHandler* mgh, SubGraphicsHandler* sgh)
{
  m_mgh = mgh;
  m_sgh = sgh;
  
  cwdsInit(0, 0);
  
  m_mode = None;  
}

/*
 * Destructor
 */
CellWriterEngine::~CellWriterEngine()
{
}

/*
 * Check if in training mode
 */
bool CellWriterEngine::IsInTrainingMode()
{
  return (m_mode == Training);
} 

/*
 * Check if in recognize mode
 */
bool CellWriterEngine::IsInRecognizeMode()
{
  return (m_mode == Recognize);
} 

/*
 * Begin training mode for the requested char
 */
void CellWriterEngine::BeginTrainingMode(int ch, int cellX, int cellY, int cellW, int cellH, unsigned short color)
{
  m_cellX = cellX;
  m_cellY = cellY;
  m_color = color;
  
  cwdsBeginTrainingMode(ch, cellW, cellH);
  
  m_mode = Training;
}

/*
 * Begin recognize mode
 */
void CellWriterEngine::BeginRecognizeMode(int cellX, int cellY, int cellW, int cellH, unsigned short color)
{
  m_cellX = cellX;
  m_cellY = cellY;
  m_color = color;
  
  cwdsBeginRecognizeMode(cellW, cellH);
  
  m_mode = Recognize;
}
  
/*
 * Button depressed in cell area
 */
void CellWriterEngine::NotifyButtonPressEvent(int x, int y)
{
  render_point(false, m_cellX + x, m_cellY + y, m_color);
  
  m_lastX = x;
  m_lastY = y;
  
  cwdsNotifyPenDownEvent(x, y);
}

/*
 * Motion with button depressed in cell area
 */
void CellWriterEngine::NotifyMotionEvent(int x, int y)
{
  render_segment(false, m_cellX + m_lastX, m_cellY + m_lastY, m_cellX + x, m_cellY + y, m_color);
  
  m_lastX = x;
  m_lastY = y;
  
  cwdsNotifyPenMoveEvent(x, y);
}

/*
 * Button released
 */
void CellWriterEngine::NotifyButtonReleaseEvent()
{
  cwdsNotifyPenUpEvent();
}

/*
 * End current training mode
 */
void CellWriterEngine::EndTrainingMode()
{
  m_mode = None;

  cwdsEndTrainingMode();
}

/*
 * Cancel current training mode
 */
void CellWriterEngine::CancelTrainingMode()
{
  m_mode = None;

  cwdsCancelTrainingMode();
}

/*
 * End current recognize mode
 */
int CellWriterEngine::EndRecognizeMode()
{
  m_mode = None;

  return (cwdsEndRecognizeMode());
}

/*
 * Cancel current recognize mode
 */
void CellWriterEngine::CancelRecognizeMode()
{
  m_mode = None;

  cwdsCancelRecognizeMode();
}

/*
 * Get the number of samples for the requested char
 */
int CellWriterEngine::GetNumSamples(int ch)
{
  return (cwdsGetNumSamples(ch));
}

/*
 * Reset the samples for the requested char
 */
void CellWriterEngine::ResetSamples(int ch)
{
  cwdsResetSamples(ch);
}

/*
 * Draw the requested sample for the requested char
 */
void CellWriterEngine::DrawSample(int ch, int nSample, bool mainDisplay, int cellX, int cellY, int cellW, int cellH, unsigned short color)
{
  int ncmd = cwdsGetSampleNumGraphicsCommands(ch, nSample);
  
  for (int i=0; i<ncmd; i++)
  {
    int x1, y1, x2, y2;
    
    cwdsGetSampleGraphicCommand(ch, nSample, i, cellW, cellH, &x1, &y1, &x2, &y2);
    
    if (x2 == -1)
      render_point(mainDisplay, cellX + x1, cellY + y1, color);
    else
      render_segment(mainDisplay, cellX + x1, cellY + y1, cellX + x2, cellY + y2, color);
  }
}

/*
 * Return the recognition time for the last recognized char
 */
int CellWriterEngine::GetRecognitionTime()
{
  return (cwdsGetLastRecognitionTime());
}

/*
 * Get the max number of recognition alternatives
 */
int CellWriterEngine::GetMaxRecognitionAlts()
{
  return (cwdsGetMaxRecognitionAlts());
}
  
/*
 * Get the number of recognition alternatives
 */
int CellWriterEngine::GetNumRecognitionAlts()
{
  return (cwdsGetLastRecognitionNumAlts());
}

/*
 * Get the nth recognition alternative informations
 */
void CellWriterEngine::GetRecognitionAltInfo(int nAlt, int *ch, int *nSample, int *pct)
{
  cwdsGetLastRecognitionAltInfo(nAlt, ch, nSample, pct);
}

/*
 * Save samples
 */
void CellWriterEngine::SaveSamples(char *fileName)
{
  cwdsSaveSamples(fileName);
}

/*
 * Render requested point
 */
void CellWriterEngine::render_point(bool mainDisplay, int x, int y, unsigned short color)
{
  GraphicsHandler* gh = mainDisplay ? (GraphicsHandler *)m_mgh : (GraphicsHandler *)m_sgh;
  
  gh->DrawPoint(x, y, color);

  gh->DrawPoint(x+1, y, color);
  gh->DrawPoint(x+1, y+1, color);
  gh->DrawPoint(x, y+1, color);
}

/*
 * Render requested segment
 */
void CellWriterEngine::render_segment(bool mainDisplay, int x1, int y1, int x2, int y2, unsigned short color)
{
  GraphicsHandler* gh = mainDisplay ? (GraphicsHandler *)m_mgh : (GraphicsHandler *)m_sgh;
  
  gh->DrawLine(x1, y1, x2, y2, color);
  
  gh->DrawLine(x1+1, y1, x2+1, y2, color);
  gh->DrawLine(x1, y1+1, x2, y2+1, color);
}

/*
 * Enable/disable the requested sample
 */
void CellWriterEngine::EnableSample(int ch, bool bEnable)
{
  if (bEnable)
    cwdsEnableSymbol(ch);
  else
    cwdsDisableSymbol(ch);
}

/*
 * Enable samples in requeste range
 */
void CellWriterEngine::EnableSampleRange(int chFrom, int chTo, bool bEnable)
{
  if (bEnable)
    cwdsEnableSymbolRange(chFrom, chTo);
  else
    cwdsDisableSymbolRange(chFrom, chTo);
}
