
/* CellWriterForm implementation 

Copyright (C) 2009 Roberto Navone <ron@object65.it>
 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2
of the License.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
*/

#include <nds.h>
#include <stdio.h>

#include "colors.h"
#include "ButtonsHandler.h"
#define CELLSCRDEFS
#include "CellWriterForm.h"

struct cellCache
{
  int c;
  unsigned short color;
  void *img;
};

#define CELLCACHESIZE 140

static bool sCellCacheInit = false;
static cellCache sCellCache[CELLCACHESIZE];

/*
 * Constructor
 */
CellWriterForm::CellWriterForm(CellWriterEngine* eng, MainGraphicsHandler* mgh, SubGraphicsHandler* sgh, TimersHandler* th)
{
  if (!sCellCacheInit)
  {
    for (int i=0; i<CELLCACHESIZE; i++)
      sCellCache[i].c = -1;
  }
  
  m_eng = eng;
  m_mgh = mgh;
  m_sgh = sgh;
  m_th = th;
  
  m_bh = new ButtonsHandler(m_sgh);
  
  m_cbxh = new CheckBoxesHandler(m_sgh);
}

/*
 * Destructor
 */
CellWriterForm::~CellWriterForm()
{
  delete m_cbxh;
  delete m_bh;
}
  
/*
 * Run the form
 */
CellWriterForm::RunResult CellWriterForm::Run()
{
  /* Clear the screen */
  
  m_mgh->ClearScreen(SCR_BACK);
  m_sgh->ClearScreen(SCR_BACK);
  
  /* Initialize the form */
  
  InitForm();
  
  //DrawCell();  
    
  /* Execute the form */
  
  while (true)
  {
    int cmd = GetCommand();
    
    if (IsExitCommand(cmd))
      return (EXIT);
      
    if (IsChangeModeCommand(cmd))
      return (CHANGEMODE);
      
    HandleCommand(cmd);
  }
}

/*
 * Get a command
 */
int CellWriterForm::GetCommand()
{
  bool bPenDown = false;
  bool bDrawing = false;
  
  int lastX = 0;
  int lastY = 0;
  
  int stout = GetSamplingTimeout();
  
  touchPosition touch;
  
  Timer* tmr = 0;
  
  while (true)
  {
    swiWaitForVBlank();
    
    scanKeys();
    
    if (keysHeld() & KEY_TOUCH)
    {
      if (tmr != 0)
      {
        delete tmr;
        tmr = 0;
      }
      
      touch = touchReadXY();
      
      if (touch.px != 0  &&  touch.py != 0)
      {
        if (!bPenDown)
        {
          /* Handle begin touch */
          
          bPenDown = true;
          
          if (IsInCellArea(touch.px, touch.py))
          {
            /* Start/continue drawing */
            
            //m_gh->DrawPoint(touch.px, touch.py, CELL_DRAW);

            lastX = touch.px;
            lastY = touch.py;
            
            bDrawing = true;
            
            /* Notify event to CellWriter */
            
            HandleButtonPressEvent(lastX, lastY);
          }
          else
          {
            /***/
            
            if (bDrawing)
            {              
              HandleFinishTimeout();
              
              return (GetTimeoutCommand());
            }
          }

          Button* btn = m_bh->CheckButtonAt(touch.px, touch.py);
          
          if (btn != 0)
          {
            /* A button has been clicked */
            
            btn->Push();
            
            WaitTouchRelease();
            
            btn->Release();
            
            return (btn->GetId());
          }
          
          CheckBox* cbx = m_cbxh->CheckCheckBoxAt(touch.px, touch.py);
          
          if (cbx != 0)
          {
            /* A checkbox has been clicked */
            
            cbx->SetCheck(!cbx->IsChecked());
            
            WaitTouchRelease();
            
            return (cbx->GetId());
          }
        }
        else
        {
          /* Handle touch continuation */
          
          if (bDrawing)
          {
            if (IsInCellArea(touch.px, touch.py))
            {
              if (touch.px != lastX  ||  touch.py != lastY)
              {
                /* Continue drawing */
                
                //m_gh->DrawLine(lastX, lastY, touch.px, touch.py, CELL_DRAW);
                
                lastX = touch.px;
                lastY = touch.py;
            
                /* Notify event to CellWriter */
            
                HandleMotionNotifyEvent(lastX, lastY);
              }
            }
            //else
            //{
            //  /***/
            //  
            //  HandleButtonReleaseEvent();
            //  HandleFinishTimeout();
            //  
            //  return (GetTimeoutCommand());
            //}
          }
        }
      }
    }
    else
    {
      /* Handle touch release */
      
      bPenDown = false;
      
      if (bDrawing  &&  tmr == 0)
      {
        /* Start exit timer */
        
        tmr = m_th->CreateTimer(stout, false);
        
        //bDrawing = false;

        /* Notify event to CellWriter */
    
        HandleButtonReleaseEvent();
      }
      
      if (tmr != 0)
        if (tmr->IsElapsed())
        {
          /* Timer after draw elapsed */
          
          delete tmr;

          /* Notify event to CellWriter */
    
          HandleFinishTimeout();
          
          return (GetTimeoutCommand());
        }
    }
  }
}

/*
 * Wait touch release
 */
void CellWriterForm::WaitTouchRelease()
{
  while (true)
  {
    scanKeys();
    
    if ((keysHeld() & KEY_TOUCH) == 0)
      return;
      
    swiWaitForVBlank();
  }
}

/*
 * Draw a character cell at requested position
 */
void CellWriterForm::DrawCell(GraphicsHandler* gh, int x, int y)
{
  gh->DrawRectangle(x, y, (x + W_CELL), (y + H_UPCELL), CELL_UPBACK, true);
  gh->DrawRectangle(x, (y + H_UPCELL), (x + W_CELL), (y + H_UPCELL + H_DNCELL), CELL_DNBACK, true);
}

/*
 * Draw a character cell at requested position
 */
void CellWriterForm::DrawCell(GraphicsHandler* gh, int x, int y, int c, unsigned short color)
{
  /* Search in cache */
  
  for (int i=0; i<CELLCACHESIZE; i++)
  {
    if (sCellCache[i].c == c  &&  sCellCache[i].color == color)
    { 
      /* Draw from cache */
      
      gh->WriteBackground(sCellCache[i].img, x, y, W_CELL, H_CELL);
      
      return;
    }
  }
  
  /* First the cell */
  
  DrawCell(gh, x, y);
  
  /* Then the content */
  
  int nCustom = -1;
  
  if (c >= 0x7F)
  {
    nCustom = c - 0x7F;
    
    c = '?';
  }
  
  char txt[4];
  txt[0] = c < 0x7F ? c : '?';
  txt[1] = '\0';
  
  int h = H_UPCELL + H_DNCELL - 3;
  
  while (h > 0)
  {
    int w = gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_tuffybold, txt, h);
    
    if (w > W_CELL)
    {
      h--;
      continue;
    }
  
    gh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, txt, (x + (W_CELL - w) / 2), (y + H_UPCELL), h, color);
    
    break;
  }
  
  /* Draw custom symbol number */
  
  if (nCustom >= 0)
  {
    sprintf(txt, "%d", nCustom);
    
    h = H_DNCELL;
    
    int w = gh->GetTextTrueTypeWidth(GraphicsHandler::TTF_tuffybold, txt, h);
    
    gh->DrawTextTrueType(GraphicsHandler::TTF_tuffybold, txt, (x + (W_CELL - w) / 2), (y + H_UPCELL + H_DNCELL), h, color);
  }
  
  /* Put in cache for the next time */
  
  for (int i=0; i<CELLCACHESIZE; i++)
  {
    if (sCellCache[i].c == -1)
    { 
      sCellCache[i].c = c;
      sCellCache[i].color = color;
      
      sCellCache[i].img = gh->SaveBackground(x, y, W_CELL, H_CELL);
      
      break;
    }
  }
  
}
